"""
Overview:
    Meta information for cheesechaser package.
"""

#: Title of this project (should be `cheesechaser`).
__TITLE__ = 'cheesechaser'

#: Version of this project.
__VERSION__ = '0.1.1'

#: Short description of the project, will be included in ``setup.py``.
__DESCRIPTION__ = 'Swiftly get tons of images from indexed tars on Huggingface.'

#: Author of this project.
__AUTHOR__ = 'narugo1992'

#: Email of the authors'.
__AUTHOR_EMAIL__ = 'narugo992@gmail.com'
